#!/usr/bin/env python

import math
from optparse import OptionParser
import dxfwrite
from dxfwrite import DXFEngine as dxf



parser = OptionParser("")
parser.add_option("--name", default="heat-exchanger")
parser.add_option("--Nx", default=10, type=int)
parser.add_option("--Ny", default=5, type=int)
#parser.add_option("--x-offset", default=0, type=float)
#parser.add_option("--y-offset", default=5, type=int)
parser.add_option("--height", default=500, type=float)
parser.add_option("--pipe-radius", default=40, type=float)
parser.add_option("--pipe-thickness", default=2, type=float)
parser.add_option("--pipe-spacing", default=20, type=float)
parser.add_option("--thickness", default=2, type=float)
parser.add_option("--edge", default=1., type=float)
parser.add_option("--pin-every", default=3, type=float)
parser.add_option("--pin-width", default=20, type=float)
parser.add_option("--separators", default=3, type=float)
parser.add_option("--tolerance", default=0.25, type=float)
parser.add_option("--single", action="store_true", default=False)

opts, args = parser.parse_args()

name = opts.name
Nx = opts.Nx
Ny = opts.Ny
pipe_spacing = opts.pipe_spacing
pipe_radius = opts.pipe_radius
pipe_thickness = opts.pipe_thickness

spacing =  pipe_spacing + pipe_radius * 2
offset = pipe_spacing# /2
#x_offset = opts.x_offset
width = Nx * spacing + pipe_spacing
depth = Ny * spacing + pipe_spacing
height = opts.height
thickness = opts.thickness
edge = opts.edge
pin_every = opts.pin_every
pin_width = opts.pin_width
tolerance = opts.tolerance
separators = opts.separators

print "width = %fmm" % width, "(%fmm including edge)" % (width + edge*2)
print "depth = %fmm" % depth, "(%fmm including edge)" % (depth + edge*2)


area = 2 * math.pi * ((pipe_radius - pipe_thickness)/1000.)  * height/1000. * Nx * Ny
print "area (inner) = %f m^2" % (area) 

volume_box = (width - thickness * 2) * (height - thickness * 2) * (depth - thickness * 2) / 100**3 # in liters
volume_pipes = math.pi * pipe_radius**2 * height / 100**3 * Nx * Ny # in liters
print "volume"
print "\tin box = %f liters" % volume_box
print "\tin pipes = %f liters" % volume_pipes
print "\tnett = %f liters" % (volume_box-volume_pipes)


# DRAW TOP
single_file = opts.single
if single_file:
	filename = name +'.dxf'
	print "writing", filename
	drawing = dxf.drawing(filename)
	drawing.add_layer('LINES')
else:
	filename = name +'-top-bottom.dxf'
	print "writing", filename
	drawing = dxf.drawing(filename)
	drawing.add_layer('LINES')

x0, y0 = 0, 0
x1, y1 = x0 + width, y0 + depth

for x in range(Nx):
	for y in range(Ny):
		center = x0 + offset + pipe_radius + spacing * (x), \
					y0 + offset + pipe_radius + spacing * (y)				
		drawing.add(dxf.circle(radius=pipe_radius+tolerance, center=center))
# add some extra spacing for welding?
x0 -= edge
y0 -= edge
x1 += edge
y1 += edge
drawing.add(dxf.polyline([(x0, y0), (x1, y0), (x1, y1), (x0, y1)], color=7, layer='LINES', flags=dxfwrite.POLYLINE_CLOSED))
drawing.save()
x0, y0 = 0, 0
x1, y1 = x0 + width, y0 + depth

# MIDDLE PARTS

if single_file:
	x0, y0 = 0, depth+20
	x1, y1 = x0 + width, y0 + depth
else:
	filename = name +'-middle.dxf'
	print "writing", filename
	drawing = dxf.drawing(filename)
	drawing.add_layer('LINES')
	x0, y0 = 0, 0
	x1, y1 = x0 + width, y0 + depth

for x in range(Nx-1):
	for y in range(Ny):
		center = x0 + offset + pipe_radius + spacing * (x), \
					y0 + offset + pipe_radius + spacing * (y)					
		drawing.add(dxf.circle(radius=pipe_radius+tolerance, center=center))

points = []

points.append((x0+thickness, y0+thickness))
pins_indices_x = []
for x in range(Nx//2):
	#print x, Nx-2-x
	if ((x-1) % pin_every) == 0:
		pins_indices_x.append(x)
		pins_indices_x.append((Nx-2-x))
pins_indices_y = []
for y in range(Ny//2):
	#print y, Ny-2-x
	if ((y) % pin_every) == 0:
		pins_indices_y.append(y)
		pins_indices_y.append((Ny-2-y))

pins_indices_x = list(set(pins_indices_x))
pins_indices_x.sort()
pins_indices_y = list(set(pins_indices_y))
pins_indices_y.sort()
#print pins_indices_x
#print pins_indices_y

# from bottom left to bottom right
for x in range(Nx):
	if x in pins_indices_x:
		offset_pin = x0 + offset + pipe_radius*2 + spacing * x + pipe_spacing / 2
		points.append((x0+offset_pin - pin_width/2, y0+thickness))
		points.append((x0+offset_pin - pin_width/2, y0-edge))
		points.append((x0+offset_pin + pin_width/2, y0-edge))
		points.append((x0+offset_pin + pin_width/2, y0+thickness))
		
# from bottom right to top right
points.append((x1-thickness - spacing, y0+thickness))
# from top right to top left
points.append((x1-thickness - spacing, y1-thickness))
for x in range(Nx)[::-1]:
	if x in pins_indices_x:
		offset_pin = x0 + offset + pipe_radius*2 + spacing * x + pipe_spacing / 2
		points.append((offset_pin + pin_width/2, y1-thickness))
		points.append((offset_pin + pin_width/2, y1+edge))
		points.append((offset_pin - pin_width/2, y1+edge))
		points.append((offset_pin - pin_width/2, y1-thickness))
points.append((x0+thickness, y1-thickness))

# from top left to bottom left
for y in range(Ny)[::-1]:
	if y in pins_indices_y:
		offset_pin = y0 + offset + pipe_radius*2 + spacing * y + pipe_spacing / 2
		points.append((x0+thickness, offset_pin + pin_width/2))
		points.append((x0-edge,      offset_pin + pin_width/2))
		points.append((x0-edge,      offset_pin - pin_width/2))
		points.append((x0+thickness, offset_pin - pin_width/2))
#
	
drawing.add(dxf.polyline(points, color=7, layer='LINES', flags=dxfwrite.POLYLINE_CLOSED))
drawing.save()


# front

if single_file:
	x0, y0 = 0, depth*2+20*2 
	x1, y1 = x0 + width, y0 + depth
else:
	filename = name +'-front.dxf'
	print "writing", filename
	drawing = dxf.drawing(filename)
	drawing.add_layer('LINES')

	x0, y0 = 0, 0
	x1, y1 = x0 + width, y0 + height
points = []
# from bottom left to bottom right
points.append((x0-edge, y0+thickness))
# from bottom right to top right
points.append((x1+edge, y0+thickness))
points.append((x1+edge, y1-thickness))
# from bottom right to top left
points.append((x0-edge, y1-thickness))
# from top left to bottom left
# auto
drawing.add(dxf.polyline(points, color=7, layer='LINES', flags=dxfwrite.POLYLINE_CLOSED))

for x in range(Nx):
	if x in pins_indices_x:
		cx = center_pin_x = x0 + offset + pipe_radius*2 + spacing * x + pipe_spacing / 2
		for y in range(separators):
			cy = center_pin_y = y0 + height/(separators+1.) * (y+1)
			points = [
				(cx-pin_width/2-tolerance, cy-thickness-tolerance),
				(cx+pin_width/2+tolerance, cy-thickness-tolerance),
				(cx+pin_width/2+tolerance, cy+thickness+tolerance),
				(cx-pin_width/2-tolerance, cy+thickness+tolerance),
			 ]
			drawing.add(dxf.polyline(points, color=7, layer='LINES', flags=dxfwrite.POLYLINE_CLOSED))
drawing.save()



# side

for side in [0,1]:
	if single_file:
		x0, y0 = width+20, (height + 20) *side 
		x1, y1 = x0 + depth, y0 + height
	else:
		filename = name +'-side-%d.dxf' % side
		print "writing", filename
		drawing = dxf.drawing(filename)
		drawing.add_layer('LINES')
		x0, y0 = 0, 0
		x1, y1 = x0 + depth, y0 + height
	points = []
	# from bottom left to bottom right
	points.append((x0+thickness, y0+thickness))
	# from bottom right to top right
	points.append((x1-thickness, y0+thickness))
	points.append((x1-thickness, y1-thickness))
	# from bottom right to top left
	points.append((x0+thickness, y1-thickness))
	# from top left to bottom left
	# auto
	drawing.add(dxf.polyline(points, color=7, layer='LINES', flags=dxfwrite.POLYLINE_CLOSED))

	for y in range(Ny):
		if y in pins_indices_y:
			cy = center_pin_y = x0 + offset + pipe_radius*2 + spacing * y + pipe_spacing / 2
			for z in range(separators):
				if ((z+side) % 2) == 0: # off or even?
					cz = center_pin_z = y0 + height/(separators+1.) * (z+1)
					points = [
						(cy-pin_width/2-tolerance, cz-thickness-tolerance),
						(cy+pin_width/2+tolerance, cz-thickness-tolerance),
						(cy+pin_width/2+tolerance, cz+thickness+tolerance),
						(cy-pin_width/2-tolerance, cz+thickness+tolerance),
					]
					drawing.add(dxf.polyline(points, color=7, layer='LINES', flags=dxfwrite.POLYLINE_CLOSED))
	drawing.save()



